@@ -17,8 +17,7 @@
 ;; for more details.
 
 ;; You should have received a copy of the GNU Lesser General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 
 ;;; Note: This library was ported from the Elisp library, so don't wonder much
@@ -60,6 +59,7 @@ Useful only for diagnosing problems.")
 (defvar *language-codes*
   '(("czech" . "cs")
     ("english" . "en")
+    ("american english" . "en-US")
     ("french" . "fr")
     ("german" . "de"))
   "Mapping of LANG values to language ISO codes.")
@@ -79,6 +79,7 @@ Useful only for diagnosing problems.")
     (voice . "VOICE")
     (rate . "RATE")
     (pitch . "PITCH")
+    (pitch_range . "PITCH_RANGE")
     (spelling-mode . "SPELLING")
     (output-module . "OUTPUT_MODULE")
     ))
@@ -97,6 +98,7 @@ Useful only for diagnosing problems.")
     (punctuation-mode
      (:none . "none")
      (:some . "some")
+     (:most . "most")
      (:all .  "all"))
     (capital-character-mode
      (:none . "none")
@@ -110,6 +112,7 @@ Useful only for diagnosing problems.")
 
 (defparameter +punctuation-modes+ '(("none" . none)
                                     ("some" . some)
+                                    ("most" . most)
                                     ("all" .  all)))
 
 (defparameter +capital-character-modes+ '(("none" .  none)
@@ -383,7 +386,7 @@ Return the opened connection on success, nil otherwise."
     ("key")
     ("quit")
     ("block" ("end"))
-    ("set" ("self" ("rate" "pitch" "voice" "language")))))
+    ("set" ("self" ("rate" "pitch" "pitch_range" "voice" "language")))))
 
 (defun block-command-p (command &optional allowed)
   (unless allowed
