@@ -17,8 +17,7 @@
    for more details.
 
    You should have received a copy of the GNU Lesser General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -209,6 +208,10 @@ SSIP_OUTPUT_DECL(text, "text")
     SSIP_SET_DECL(pitch, "pitch")
     SSIP_SET_INT_BODY(voice_pitch)
 #undef FUNC_NAME
+#define FUNC_NAME s_ssip_set_pitch_range
+    SSIP_SET_DECL(pitch_range, "pitch_range")
+    SSIP_SET_INT_BODY(voice_pitch_range)
+#undef FUNC_NAME
 #define FUNC_NAME s_ssip_set_volume
     SSIP_SET_DECL(volume, "volume")
     SSIP_SET_INT_BODY(volume)
@@ -241,6 +244,8 @@ SSIP_SET_DECL(punctuation_mode, "punctuation-mode")
 			mode = SPD_PUNCT_NONE;
 		else if (!strcmp(c_value, "some"))
 			mode = SPD_PUNCT_SOME;
+		else if (!strcmp(c_value, "most"))
+			mode = SPD_PUNCT_MOST;
 		else if (!strcmp(c_value, "all"))
 			mode = SPD_PUNCT_ALL;
 		else
