@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: nas.c,v 1.8 2006-07-11 16:12:26 hanke Exp $
  */
@@ -90,7 +88,6 @@ static AudioID *nas_open(void **pars)
 {
 	spd_nas_id_t *nas_id;
 	int ret;
-	AuBool r;
 
 	nas_id = (spd_nas_id_t *) g_malloc(sizeof(spd_nas_id_t));
 
@@ -130,7 +127,6 @@ static int nas_play(AudioID * id, AudioTrack track)
 	char *buf;
 	Sound s;
 	AuEventHandlerRec *event_handler;
-	int ret;
 	float lenght;
 	struct timeval now;
 	struct timespec timeout;
@@ -157,6 +153,7 @@ static int nas_play(AudioID * id, AudioTrack track)
 					    &nas_id->flow, NULL, NULL, NULL);
 
 	if (event_handler == NULL) {
+		pthread_mutex_unlock(&nas_id->flow_mutex);
 		fprintf(stderr,
 			"AuSoundPlayFromData failed for unknown resons.\n");
 		return -1;
@@ -186,7 +183,6 @@ static int nas_play(AudioID * id, AudioTrack track)
 
 static int nas_stop(AudioID * id)
 {
-	int ret;
 	spd_nas_id_t *nas_id = (spd_nas_id_t *) id;
 
 	if (nas_id == NULL)
