@@ -16,9 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,7 +30,6 @@
 #include <string.h>
 
 #include <glib.h>
-#include <semaphore.h>
 
 #include <picoapi.h>
 
@@ -41,11 +38,12 @@
 #include "module_utils.h"
 
 #define MODULE_NAME     "pico"
-#define MODULE_VERSION  "0.1"
+#define MODULE_VERSION  "0.2"
 
 DECLARE_DEBUG();
 
-#define MAX_OUTBUF_SIZE		(128)
+#define MIN_OUTBUF_SIZE		(1600)
+#define MAX_OUTBUF_SIZE		(3200)
 #define PICO_MEM_SIZE			(10000000)
 
 #define PICO_VOICE_SPEED_MIN		(20)
@@ -87,12 +85,12 @@ static const char *picoInternalSgLingware[] = {
 };
 
 static const SPDVoice pico_voices[] = {
-	{"samantha", "en", "en-US"},
-	{"serena", "en", "en-GB"},
-	{"sabrina", "de", "de-DE"},
-	{"isabel", "es", "es-ES"},
-	{"virginie", "fr", "fr-FR"},
-	{"silvia", "it", "it-IT"}
+	{"samantha", "en-US", NULL},
+	{"serena", "en-GB", NULL},
+	{"sabrina", "de-DE", NULL},
+	{"isabel", "es-ES", NULL},
+	{"virginie", "fr-FR", NULL},
+	{"silvia", "it-IT", NULL}
 };
 
 static const SPDVoice *pico_voices_list[] = {
@@ -105,10 +103,6 @@ static const SPDVoice *pico_voices_list[] = {
 	NULL
 };
 
-static GThread *pico_play_thread;
-static sem_t pico_play_semaphore;
-static sem_t pico_idle_semaphore;
-
 enum states { STATE_IDLE, STATE_PLAY, STATE_PAUSE, STATE_STOP, STATE_CLOSE };
 static enum states pico_state;
 
@@ -161,6 +155,7 @@ static int pico_set_pitch(signed int value)
 static int pico_process_tts(void)
 {
 	pico_Int16 bytes_sent, bytes_recv, text_remaining, out_data_type;
+	pico_Int16 bytes_stored;
 	int ret, getstatus;
 	short outbuf[MAX_OUTBUF_SIZE];
 	pico_Retstring outMessage;
@@ -178,6 +173,9 @@ static int pico_process_tts(void)
 
 	/* synthesis loop   */
 	while (text_remaining) {
+		/* Process server events in case we were told to stop in between */
+		module_process(STDIN_FILENO, 0);
+
 		/* Feed the text into the engine.   */
 		if ((ret = pico_putTextUtf8(picoEngine, buf, text_remaining,
 					    &bytes_sent))) {
@@ -187,15 +185,19 @@ static int pico_process_tts(void)
 			    outMessage);
 			return -1;
 		}
+		DBG(MODULE_NAME " Sent %d bytes\n", bytes_sent);
 
 		text_remaining -= bytes_sent;
 		buf += bytes_sent;
+		bytes_stored = 0;
 
 		do {
 			/* Retrieve the samples and add them to the buffer.
 			   SVOX pico TTS sample rate is 16K */
-			getstatus = pico_getData(picoEngine, (void *)outbuf,
-						 MAX_OUTBUF_SIZE, &bytes_recv,
+			getstatus = pico_getData(picoEngine,
+						 (void *)outbuf + bytes_stored,
+						 MAX_OUTBUF_SIZE - bytes_stored,
+						 &bytes_recv,
 						 &out_data_type);
 			if ((getstatus != PICO_STEP_BUSY)
 			    && (getstatus != PICO_STEP_IDLE)) {
@@ -207,11 +209,12 @@ static int pico_process_tts(void)
 				return -1;
 			}
 
-			if (bytes_recv) {
-				track.num_samples = bytes_recv / 2;
-				track.samples =
-				    (short *)g_memdup((gconstpointer) outbuf,
-						      bytes_recv);
+			bytes_stored += bytes_recv;
+
+			if (bytes_stored >= MIN_OUTBUF_SIZE
+					|| PICO_STEP_BUSY != getstatus) {
+				track.num_samples = bytes_stored / 2;
+				track.samples = outbuf;
 				track.num_channels = 1;
 				track.sample_rate = PICO_SAMPLE_RATE;
 				track.bits = 16;
@@ -219,13 +222,10 @@ static int pico_process_tts(void)
 				    ": Sending %i samples to audio.",
 				    track.num_samples);
 
-				if (module_tts_output(track, format) < 0) {
-					DBG(MODULE_NAME
-					    "Can't play track for unknown reason.");
-					return -1;
-				}
+				module_tts_output_server(&track, format);
+				bytes_stored = 0;
 			}
-			if (g_atomic_int_get(&pico_state) != STATE_PLAY) {
+			if (pico_state != STATE_PLAY) {
 				text_remaining = 0;
 				break;
 			}
@@ -237,50 +237,6 @@ static int pico_process_tts(void)
 	return 0;
 }
 
-/* Playback thread. */
-static gpointer pico_play_func(gpointer nothing)
-{
-	DBG(MODULE_NAME ": Playback thread starting");
-
-	set_speaking_thread_parameters();
-
-	while (g_atomic_int_get(&pico_state) != STATE_CLOSE) {
-
-		sem_wait(&pico_play_semaphore);
-		if (g_atomic_int_get(&pico_state) != STATE_PLAY)
-			continue;
-
-		DBG(MODULE_NAME ": Sending to TTS engine");
-		module_report_event_begin();
-
-		if (0 != pico_process_tts()) {
-			DBG(MODULE_NAME ": ERROR in TTS");
-		}
-
-		if (g_atomic_int_get(&pico_state) == STATE_PLAY) {
-			module_report_event_end();
-			g_atomic_int_set(&pico_state, STATE_IDLE);
-		}
-
-		if (g_atomic_int_get(&pico_state) == STATE_STOP) {
-			module_report_event_stop();
-			g_atomic_int_set(&pico_state, STATE_IDLE);
-			sem_post(&pico_idle_semaphore);
-
-		}
-
-		if (g_atomic_int_get(&pico_state) == STATE_PAUSE) {
-			module_report_event_pause();
-			g_atomic_int_set(&pico_state, STATE_IDLE);
-			sem_post(&pico_idle_semaphore);
-		}
-
-		DBG(MODULE_NAME ": state %d", pico_state);
-
-	}
-	return 0;
-}
-
 /* Public functions */
 int module_load(void)
 {
@@ -300,8 +256,9 @@ int pico_init_voice(int voice_index)
 				 PICO_MAX_FILE_NAME_SIZE];
 	pico_Char picoSgFileName[PICO_MAX_DATAPATH_NAME_SIZE +
 				 PICO_MAX_FILE_NAME_SIZE];
-	pico_Char picoTaResourceName[PICO_MAX_RESOURCE_NAME_SIZE];
-	pico_Char picoSgResourceName[PICO_MAX_RESOURCE_NAME_SIZE];
+
+	pico_Retstring picoTaResourceName;
+	pico_Retstring picoSgResourceName;
 
 	/* Load the text analysis Lingware resource file.   */
 	strcpy((char *)picoTaFileName, PicoLingwarePath);
@@ -331,7 +288,7 @@ int pico_init_voice(int voice_index)
 
 	/* Get the text analysis resource name.     */
 	if ((ret = pico_getResourceName(picoSystem, picoTaResource,
-					(char *)picoTaResourceName))) {
+					picoTaResourceName))) {
 		pico_getSystemStatusMessage(picoSystem, ret, outMessage);
 		DBG(MODULE_NAME
 		    "Cannot get TA resource name (%i): %s\n", ret, outMessage);
@@ -340,7 +297,7 @@ int pico_init_voice(int voice_index)
 
 	/* Get the signal generation resource name. */
 	if ((ret = pico_getResourceName(picoSystem, picoSgResource,
-					(char *)picoSgResourceName))) {
+					picoSgResourceName))) {
 		pico_getSystemStatusMessage(picoSystem, ret, outMessage);
 		DBG(MODULE_NAME
 		    "Cannot get SG resource name (%i): %s\n", ret, outMessage);
@@ -362,6 +319,7 @@ int pico_init_voice(int voice_index)
 						     (const pico_Char *)
 						     pico_voices
 						     [voice_index].name,
+						     (pico_Char *)
 						     picoTaResourceName))) {
 		pico_getSystemStatusMessage(picoSystem, ret, outMessage);
 		DBG(MODULE_NAME
@@ -375,6 +333,7 @@ int pico_init_voice(int voice_index)
 						     (const pico_Char *)
 						     pico_voices
 						     [voice_index].name,
+						     (pico_Char *)
 						     picoSgResourceName))) {
 		pico_getSystemStatusMessage(picoSystem, ret, outMessage);
 		DBG(MODULE_NAME
@@ -391,23 +350,8 @@ int module_init(char **status_info)
 	int ret, i;
 	pico_Retstring outMessage;
 	void *pmem;
-	GError *error = NULL;
 
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-
-	sem_init(&pico_play_semaphore, 0, 0);
-	sem_init(&pico_idle_semaphore, 0, 0);
-
-	if ((pico_play_thread = g_thread_create((GThreadFunc) pico_play_func,
-						NULL, TRUE, &error)) == NULL) {
-		*status_info = g_strdup_printf(MODULE_NAME
-					       "Failed to create a play thread : %s\n",
-					       error->message);
-		DBG(MODULE_NAME ": %s", *status_info);
-		g_error_free(error);
-		return -1;
-	}
+	module_audio_set_server();
 
 	pmem = g_malloc(PICO_MEM_SIZE);
 	if ((ret = pico_initialize(pmem, PICO_MEM_SIZE, &picoSystem))) {
@@ -443,13 +387,13 @@ int module_init(char **status_info)
 
 	*status_info = g_strdup(MODULE_NAME ": Initialized successfully.");
 
-	g_atomic_int_set(&pico_state, STATE_IDLE);
+	pico_state = STATE_IDLE;
 	return 0;
 }
 
 SPDVoice **module_list_voices(void)
 {
-	return pico_voices_list;
+	return (SPDVoice **)pico_voices_list;
 }
 
 void pico_set_synthesis_voice(char *voice_name)
@@ -472,6 +416,17 @@ void pico_set_synthesis_voice(char *voice_name)
 		DBG(MODULE_NAME
 		    "Cannot create a new pico engine (%i): %s\n", ret,
 		    outMessage);
+		/* Try to fallback to english */
+		if ((ret = pico_newEngine(picoSystem,
+					  (const pico_Char *)pico_voices[0].name,
+					  &picoEngine))) {
+			pico_getSystemStatusMessage(picoSystem, ret, outMessage);
+			DBG(MODULE_NAME
+			    "Cannot create default english pico engine (%i): %s\n", ret,
+			    outMessage);
+			return;
+		}
+
 		return;
 	}
 
@@ -481,10 +436,18 @@ void pico_set_synthesis_voice(char *voice_name)
 static void pico_set_language(char *lang)
 {
 	int i;
+	DBG(MODULE_NAME "setting language %s", lang);
 
 	/* get voice name based on language */
 	for (i = 0; i < sizeof(pico_voices) / sizeof(SPDVoice); i++) {
-		if (!strcmp(pico_voices[i].language, lang)) {
+		if (!strcasecmp(pico_voices[i].language, lang)) {
+			pico_set_synthesis_voice(pico_voices[i].name);
+			return;
+		}
+	}
+	/* get voice name based on main part of language */
+	for (i = 0; i < sizeof(pico_voices) / sizeof(SPDVoice); i++) {
+		if (!strncasecmp(pico_voices[i].language, lang, 2)) {
 			pico_set_synthesis_voice(pico_voices[i].name);
 			return;
 		}
@@ -492,15 +455,16 @@ static void pico_set_language(char *lang)
 	return;
 }
 
-int module_speak(char *data, size_t bytes, SPDMessageType msgtype)
+void module_speak_sync(const char *data, size_t bytes, SPDMessageType msgtype)
 {
 	int value;
 	static pico_Char *tmp;
 
-	if (g_atomic_int_get(&pico_state) != STATE_IDLE) {
+	if (pico_state != STATE_IDLE) {
 		DBG(MODULE_NAME
 		    ": module still speaking state = %d", pico_state);
-		return 0;
+		module_speak_error();
+		return;
 	}
 
 	/* Setting speech parameters. */
@@ -536,6 +500,7 @@ int module_speak(char *data, size_t bytes, SPDMessageType msgtype)
 		g_free(tmp);
 	}
 
+	/* TODO: use a generic engine for SPELL, CHAR, KEY */
 	/*      switch (msgtype) {
 	   case SPD_MSGTYPE_CHAR:
 	   case SPD_MSGTYPE_KEY:
@@ -547,9 +512,20 @@ int module_speak(char *data, size_t bytes, SPDMessageType msgtype)
 	   break;
 	   }
 	 */
-	g_atomic_int_set(&pico_state, STATE_PLAY);
-	sem_post(&pico_play_semaphore);
-	return bytes;
+	pico_state = STATE_PLAY;
+
+	module_speak_ok();
+
+	DBG(MODULE_NAME ": Sending to TTS engine");
+	module_report_event_begin();
+
+	if (0 != pico_process_tts()) {
+		DBG(MODULE_NAME ": ERROR in TTS");
+	}
+
+	module_report_event_end();
+
+	pico_state = STATE_IDLE;
 }
 
 int module_stop(void)
@@ -557,13 +533,12 @@ int module_stop(void)
 	pico_Status ret;
 	pico_Retstring outMessage;
 
-	if (g_atomic_int_get(&pico_state) != STATE_PLAY) {
+	if (pico_state != STATE_PLAY) {
 		DBG(MODULE_NAME ": STOP called when not in PLAY state");
 		return -1;
 	}
 
-	g_atomic_int_set(&pico_state, STATE_STOP);
-	sem_wait(&pico_idle_semaphore);
+	pico_state = STATE_STOP;
 
 	/* reset Pico engine. */
 	if ((ret = pico_resetEngine(picoEngine, PICO_RESET_SOFT))) {
@@ -581,13 +556,12 @@ size_t module_pause(void)
 	pico_Status ret;
 	pico_Retstring outMessage;
 
-	if (g_atomic_int_get(&pico_state) != STATE_PLAY) {
+	if (pico_state != STATE_PLAY) {
 		DBG(MODULE_NAME ": PAUSE called when not in PLAY state");
 		return -1;
 	}
 
-	g_atomic_int_set(&pico_state, STATE_PAUSE);
-	sem_wait(&pico_idle_semaphore);
+	pico_state = STATE_PAUSE;
 
 	/* reset Pico engine. */
 	if ((ret = pico_resetEngine(picoEngine, PICO_RESET_SOFT))) {
@@ -602,19 +576,12 @@ size_t module_pause(void)
 
 int module_close(void)
 {
-
-	g_atomic_int_set(&pico_state, STATE_CLOSE);
-	sem_post(&pico_play_semaphore);
-
-	g_thread_join(pico_play_thread);
+	pico_state = STATE_CLOSE;
 
 	if (picoSystem) {
 		pico_terminate(&picoSystem);
 		picoSystem = NULL;
 	}
 
-	sem_destroy(&pico_idle_semaphore);
-	sem_destroy(&pico_play_semaphore);
-
 	return 0;
 }
