@@ -47,6 +47,7 @@ int SPEAKING = 0;
 int poll_count;
 
 OutputModule *speaking_module;
+AudioID *module_audio_id;
 int speaking_uid;
 int speaking_gid;
 
@@ -65,27 +66,23 @@ void *speak(void *data)
 {
 	TSpeechDMessage *message = NULL;
 	int ret;
-	struct pollfd *poll_fds;	/* Descriptors to poll */
-	struct pollfd main_pfd;
-	struct pollfd helper_pfd;
+	struct pollfd poll_fds[2];	/* Descriptors to poll */
 	int revents;
 	OutputModule *output;
 
-	/* Block all signals and set thread states */
-	set_speak_thread_attributes();
+	/* Make interruptible */
+	set_speaking_thread_parameters();
 
-	poll_fds = g_malloc(2 * sizeof(struct pollfd));
+	/* main_pfd */
+	poll_fds[0].fd = speaking_pipe[0];
+	poll_fds[0].events = POLLIN;
+	poll_fds[0].revents = 0;
 
-	main_pfd.fd = speaking_pipe[0];
-	main_pfd.events = POLLIN;
-	main_pfd.revents = 0;
+	/* speak queue fd */
+	poll_fds[1].fd = -1;
+	poll_fds[1].events = POLLIN;
+	poll_fds[1].revents = 0;
 
-	helper_pfd.fd = -1;
-	helper_pfd.events = POLLIN;
-	helper_pfd.revents = 0;
-
-	poll_fds[0] = main_pfd;
-	poll_fds[1] = helper_pfd;
 	poll_count = 1;
 
 	while (1) {
@@ -114,7 +111,8 @@ void *speak(void *data)
 				} else if ((revents & POLLIN)
 					   || (revents & POLLPRI)) {
 					MSG(5,
-					    "wait_for_poll: activity on output_module");
+					    "wait_for_poll: activity on output_module: %d",
+					    poll_fds[1].revents);
 					/* Check if sb is speaking or they are all silent.
 					 * If some synthesizer is speaking, we must wait. */
 					is_sb_speaking();
@@ -261,6 +259,7 @@ void *speak(void *data)
 			if (strcmp(message->buf, normalized)) {
 				MSG(5, "text: Normalized '%s' to '%s'", message->buf, normalized);
 			}
+			g_free(message->buf);
 			message->buf = normalized;
 			insert_symbols(message, punct_missing);
 		}
@@ -291,9 +290,8 @@ void *speak(void *data)
 		SPEAKING = 1;
 
 		if (speaking_module != NULL) {
+			poll_fds[1].fd = speaking_module->pipe_speak[0];
 			poll_count = 2;
-			helper_pfd.fd = speaking_module->pipe_out[0];
-			poll_fds[1] = helper_pfd;
 		}
 
 		/* Set the id of the client who is speaking. */
@@ -647,7 +645,7 @@ int speaking_resume(int uid)
 	return 0;
 }
 
-int socket_send_msg(int fd, char *msg)
+int socket_send_msg(int fd, const char *msg)
 {
 	int ret;
 
@@ -663,7 +661,7 @@ int socket_send_msg(int fd, char *msg)
 	return 0;
 }
 
-int report_index_mark(TSpeechDMessage * msg, char *index_mark)
+int report_index_mark(TSpeechDMessage * msg, const char *index_mark)
 {
 	char *cmd;
 	int ret;
@@ -722,8 +720,10 @@ int is_sb_speaking(void)
 		settings = &(current_message->settings);
 
 		output_is_speaking(&index_mark);
-		if (index_mark == NULL)
+		if (index_mark == NULL) {
+			poll_count = 1;
 			return SPEAKING = 0;
+		}
 
 		if (!strcmp(index_mark, "no")) {
 			g_free(index_mark);
@@ -935,26 +935,6 @@ gint message_nto_speak(gconstpointer data, gconstpointer nothing)
 		return 1;
 }
 
-void set_speak_thread_attributes()
-{
-	int ret;
-	sigset_t all_signals;
-
-	ret = sigfillset(&all_signals);
-	if (ret == 0) {
-		ret = pthread_sigmask(SIG_BLOCK, &all_signals, NULL);
-		if (ret != 0)
-			MSG(1,
-			    "Can't set signal set, expect problems when terminating!");
-	} else {
-		MSG(1,
-		    "Can't fill signal set, expect problems when terminating!");
-	}
-
-	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
-	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
-}
-
 void stop_priority_except_first(SPDPriority priority)
 {
 	GList *queue;
